#!/opt/cyops-auth/.env/bin/python3
""" Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import requests, urllib3, os, argparse, json
from cshmac.requests import HmacAuth
#
# Please note that this script calls API's using localhost (variable declared 'CH_HOST').
# The environment which is executing this script may NOT have 'no_proxy' set with 'localhost'.
# In the environment in which 'https_proxy' is set and there is NO 'no_proxy' set for 'localhost',
# below API calls in this script will start failing. Hence, regardless of whether executing
# environment have 'no_proxy' set or not, define it here explicitly.
#
os.environ['no_proxy'] = "127.0.0.1, localhost"
KEY_PATH = '/opt/cyops/configs/keys/'
APPLIANCE_PUBLIC_KEY = ''
APPLIANCE_PRIVATE_KEY = ''

def read_files(key_path):
    global APPLIANCE_PUBLIC_KEY
    global APPLIANCE_PRIVATE_KEY
    with open(os.path.join(key_path, 'APPLIANCE_PRIVATE_KEY'), 'r') as private:
        APPLIANCE_PRIVATE_KEY = private.read().strip()
    with open(os.path.join(key_path, 'APPLIANCE_PUBLIC_KEY'), 'r') as public:
        APPLIANCE_PUBLIC_KEY = public.read().strip()

def get_appliance_keys():
    read_files(KEY_PATH)

def print_me(msg, silent=False):
    if silent:
        return
    print(msg)

def print_response(response):
    try:
        # First see if can be printed in JSON
        print(json.dumps(response.json(), indent=4))
    except Exception as e:
        # Object is not converted to JSON. Happens due to non-json reponse.
        try:
            # See if can be printed in string utf-8
            print(response.content.decode("utf-8"))
        except Exception as e:
            # Print content in byte.
            print(response.content)

def make_request(method, endpoint, payload=None):
    try:
        urllib3.disable_warnings()
        get_appliance_keys()
        auth = None
        response = None
        if method.lower() == "delete":
            auth = HmacAuth(endpoint, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, APPLIANCE_PUBLIC_KEY)
        else:
            auth = HmacAuth(endpoint, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, json.dumps(payload))
        if method.lower() == "delete":
            response = requests.delete(endpoint, auth=auth, verify=False)
        else:
            response = requests.request(method, endpoint, auth=auth, json=payload, verify=False)
        print_response(response)
        if response.status_code == 200 or response.status_code == 201 or response.status_code == 204:
            return True
        elif response.status_code == 503 or response.status_code == 403 or response.status_code == 404:
            return False
    except Exception as Err:
        print(Err)
        return False

def call_api(method, endpoint, payload=None, silent=False):
    if not silent:
        print_me("Calling api endpoint " + endpoint + " with method " + method)
    if not make_request(method, endpoint, payload):
        if not silent:
            print("Failed")
        exit(1)
    if not silent:
        print_me("Successfully executed API.")

def define_parser(parser):
    parser.add_argument('--method', help="REST API method. Default set to 'GET'", dest='method', type=str, required=False, default='GET')
    parser.add_argument('--endpoint', help='Full path of endpoint. Eg. https://localhost/api/query/alerts?$limit=30', dest='endpoint', type=str, required=True)
    parser.add_argument('--payload', help='Payload in json format. Eg. {"key" : "value"}', dest='payload', type=str, required=False, default=None)
    parser.add_argument('--silent',
                        help="Do not print informative statements",
                        action='store_true',
                        dest='is_silent',
                        default=False)
    
if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    define_parser(parser)
    args = parser.parse_args()
    payload = None
    try:
        if args.payload is not None:
            payload = json.loads(args.payload)
    except Exception as e:
        print("Provide payload in json format")
        exit(1)
    call_api(args.method, args.endpoint, payload, args.is_silent)
    exit(0)